﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using System.ServiceProcess;
using System.Diagnostics;



namespace KeiganMotorKM_1_Inspection
{
    public partial class Form1 : Form
    {
        public KeiganMotor_USB targetMotor;
        private bool shouldSaveMotorMeasurement = false;
        private string fileName = "test";

        public Form1()
        {
            InitializeComponent();
            Debug.Listeners.Add(new TextWriterTraceListener(Console.Out));

        }

        delegate void StringArgReturningVoidDelegate(string text);


        private void LogAppendText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (MeasTextBox.InvokeRequired)
            {
                StringArgReturningVoidDelegate d = new StringArgReturningVoidDelegate(LogAppendText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                LogTextBox.AppendText(text);
            }
        }


        private void LogSetText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (MeasTextBox.InvokeRequired)
            {
                StringArgReturningVoidDelegate d = new StringArgReturningVoidDelegate(LogSetText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                LogTextBox.Text = text;
            }
        }

        private void ErrorSetText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (ErrorTextBox.InvokeRequired)
            {
                StringArgReturningVoidDelegate d = new StringArgReturningVoidDelegate(ErrorSetText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                ErrorTextBox.Text = text;
            }
        }


        private void MeasurementSetText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (MeasTextBox.InvokeRequired)
            {
                StringArgReturningVoidDelegate d = new StringArgReturningVoidDelegate(MeasurementSetText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                MeasTextBox.Text = text;
            }
        }

        private void PIDAddText(string text)
        {
            string current = ReadPIDTableTextBox.Text;
            PIDSetText(current + text);
        }

        private void PIDSetText(string text)
        {
            // InvokeRequired required compares the thread ID of the
            // calling thread to the thread ID of the creating thread.
            // If these threads are different, it returns true.
            if (ReadPIDTableTextBox.InvokeRequired)
            {
                StringArgReturningVoidDelegate d = new StringArgReturningVoidDelegate(PIDSetText);
                this.Invoke(d, new object[] { text });
            }
            else
            {
                ReadPIDTableTextBox.Text = text;
            }
        }


        public void ReadStatus(KeiganMotor_USB motor, KeiganMotor_USB.Status status)
        {
            String enabled = (status.isEnabled) ? "isEnabled" : "isDisabled";
            String queuePaused = (status.isQueuePaused) ? "isPaused" : "isResumed";
            String motorMeasurementEnabled = (status.isMotorMeasurementEnabled) ? "isEnabled" : "isDisabled";
            String iMUMeasurementEnabled = (status.isIMUMeasurementEnabled) ? "isEnabled" : "isDisabled";
            String checkSumEnabled = (status.isCheckSumEnabled) ? "isEnabled" : "isDisabled";

            LogSetText("Motor: " + enabled + "\r\n" +
                          "Queue: " + queuePaused + "\r\n" +
                          "Motor Mesurement: " + motorMeasurementEnabled + "\r\n" +
                          "IMU Mesurement: " + iMUMeasurementEnabled + "\r\n" +
                          "CheckSum " + checkSumEnabled
                          );
        }

        public void ReceivedError(KeiganMotor_USB motor, UInt16 id, byte cmd, KeiganMotor_USB.ErrorCode code, UInt32 info)
        {
            // TODO
            ErrorSetText("ID: " + id + "\r\nCommand: 0x" + cmd.ToString("X2") + "\r\nErrorCode: " + code + "\r\nInfo: " + info);
        }

        static bool isPosArrivalTestEnabled = false;
        static UInt32 posArrivalCount = 0;

        public void ReceivedPosArrival(KeiganMotor_USB motor, UInt16 id, float arrivalPos)
        {
            if (isPosArrivalTestEnabled)
            {
                posArrivalCount++;
                Console.WriteLine("Pos Arrival Count:{0}, Pos:{1}", posArrivalCount, arrivalPos);

                if (!TargetMotorExists()) return;

                if (posArrivalCount % 2 == 0)
                {    
                    targetMotor.MoveByDistanceDegree(360);   
                } else
                {
                    targetMotor.MoveByDistanceDegree(-360);
                }
            }
        }

        public void ReadMotion(KeiganMotor_USB motor, UInt16 id, UInt32 totalPosNum, UInt32 posIndex, UInt32 posNum, float[] posArray)
        {
            Console.WriteLine("ReadMotion Index: {0}, Counts: {1} / total {2}", posIndex, posNum, totalPosNum);
            Console.WriteLine(string.Join(", ", posArray));
        }

        public void AppendMotorMeasData(String meas)
        {
            if (shouldSaveMotorMeasurement)
            {
                var append = true;
                DateTime dt = DateTime.Now;
                meas = dt + "," + meas; 
                using (var sw = new System.IO.StreamWriter(fileName, append))
                {
                    System.Console.WriteLine("Write file.");
                    sw.WriteLine(meas);
                    sw.Close();
                }
            }
        }

        public void MotorMeasurement(KeiganMotor_USB motor, float position, float velocity, float torque)
        {
            DrawMeasurementGraph();
            String meas = motor.degree.ToString("F3") + ", " + motor.rpm.ToString("F3") + ", " + torque.ToString("F3");
            String str = "degree, rpm, torque = " + meas;
            MeasurementSetText(str);

            AppendMotorMeasData(meas);


        }


        public void ReadPIDTable(KeiganMotor_USB motor, UInt16 id, byte tableIndex, KeiganMotor_USB.PIDParamType type, float value)
        {
            PIDAddText("Table: " + tableIndex + ", " + type + ", value = " + value + "\r\n");
            
        }

        public void ReadText(KeiganMotor_USB motor, String text)
        {
           
            //LogAppendText(text);
        }


        private void ConnectionButton_Click(object sender, EventArgs e)
        {
            // 接続状態の場合
            if (targetMotor != null && targetMotor.serialPort.IsOpen == true)
            {
                // シリアルポートをクローズする
                targetMotor.serialPort.Close();
                targetMotor = null; 
                // ボタン表示の変更 切断→接続
                connectionButton.Text = "Connect";
            }
            // 切断状態の場合
            else
            {
                //! オープンするシリアルポートをコンボボックスから取り出す.
                string portName = ComboBox1.SelectedItem.ToString();

                try
                {
                    //　シリアルポートをオープンする.
                    targetMotor = new KeiganMotor_USB(portName);
                    targetMotor.Open();
                    targetMotor.ReadPIDTableEvent += ReadPIDTable;

                    targetMotor.MotorMeasurementEvent += MotorMeasurement;
                    targetMotor.ReadStatusEvent += ReadStatus;
                    targetMotor.ReadTextEvent += ReadText;
                    targetMotor.ReceivedErrorEvent += ReceivedError;
                    targetMotor.ReceivedPosArrivalEvent += ReceivedPosArrival;
                    targetMotor.ReadMotionEvent += ReadMotion;

             
                    //　ボタン表示の変更 接続→切断
                    connectionButton.Text = "Disconnect";
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }


        private bool TargetMotorExists()
        {
            if(targetMotor == null){
                return false;
            } else
            {
                return true;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Update_PortNames();
        }

        private void reloadButton_Click(object sender, EventArgs e)
        {
            Update_PortNames();
        }

        private void Update_PortNames()
        {
            string[] PortList = SerialPort.GetPortNames();

            ComboBox1.Items.Clear();

            foreach (string PortName in PortList)
            {
                ComboBox1.Items.Add(PortName);
            }
            if (ComboBox1.Items.Count > 0)
            {
                ComboBox1.SelectedIndex = ComboBox1.Items.Count - 1;
            }
        }


        // Power Group
        private void EnableButton_Click(object sender, EventArgs e)
        {
            if(!TargetMotorExists()) { return;}
            targetMotor.Enable();
            targetMotor.ReadStatus();
        }

        private void DisableButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.Disable();
            targetMotor.ReadStatus();
        }


        // Speed Group
        private void SpeedButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(SpeedTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.SpeedRpm(v);
            } else
            {
                MessageBox.Show("Invalid Speed.");
            }
            
        }


        private void RunForwardButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.RunForward();
        }

        private void RunReverseButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.RunReverse();
        }


        // Position Group
        private void MoveToPositionButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(PositionTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.MoveToPositionDegree(v);
            }
            else
            {
                MessageBox.Show("Invalid Position.");
            }
        }

        private void MoveByDistanceP_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(DistanceTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.MoveByDistanceDegree(v);
            }
            else
            {
                MessageBox.Show("Invalid Position.");
            }
        }

        private void MoveByDistanceN_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(DistanceTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.MoveByDistanceDegree(-v);
            }
            else
            {
                MessageBox.Show("Invalid Position.");
            }
        }


        /*
          * Others
          */

        private void StopButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.Stop();
        }


        private void FreeButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.Free();
        }

        private void rebootButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.Reboot();
        }

        /* 
         * LED
         */
        private void RedButton_CheckedChanged(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            Console.WriteLine("RedButton_CheckedChanged");
            targetMotor.Led(KeiganMotor_USB.LedState.OnSolid, 255, 0, 0);
        }

        private void GreenButton_CheckedChanged(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            Console.WriteLine("GreenButton_CheckedChanged");
            targetMotor.Led(KeiganMotor_USB.LedState.OnSolid, 0, 255, 0);
        }

        private void BlueButton_CheckedChanged(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.Led(KeiganMotor_USB.LedState.OnSolid, 0, 0, 255);
        }

        private void OrangeButton_CheckedChanged(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.Led(KeiganMotor_USB.LedState.OnSolid, 255, 165, 0);
        }

        private void YellowButton_CheckedChanged(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.Led(KeiganMotor_USB.LedState.OnSolid, 255, 255, 0);
        }

        private void PurpleButton_CheckedChanged(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.Led(KeiganMotor_USB.LedState.OnSolid, 128, 0, 128);
        }


        /*
         * Teaching Playback of Motion 
         * 
         */

        // Motion Index
        private UInt16 motionIndex = 0;

        private void motionUpDown_ValueChanged(object sender, EventArgs e)
        {
            motionIndex = (UInt16)motionUpDown.Value;
        }

        // Erase Motion
        private void eraseMotionButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.EraseMotion(motionIndex);
        }


        private void eraseAllMotionButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.EraseAllMotion();
        }

        // Teaching Motion
        private void prepareTeachingButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.PrepareTeachingMotion(motionIndex, 0);
        }

        private void teachingStartButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.StartTeachingMotionFromPrep();
        }

        private void stopTeachingButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.StopTeachingMotion();
        }

        private void TeachingIntervalSetButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            UInt32 v;
            bool valid = UInt32.TryParse(TeachingIntervalTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.TeachingInterval(v);
            }
            else
            {
                MessageBox.Show("Invalid Teaching Interval.");
            }
        }

        private float ampDeg = 30;
        private void WritePosTestButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }


            float freq = 1.0f; // [Hz] 1s に 振動する回数 
            float period = 1 / freq;
            float amp = (float)(ampDeg * Math.PI / 180); // deg to radian
            int len = (int)(period * 1000 / 10); // ティーチングサンプリング間隔は 10ms（デフォルト） 
            for (int t = 0; t < len; t++)
            {
                float p = (float)(amp * Math.Cos(2 * Math.PI * t / len));
                targetMotor.WriteMotionPosition(p);
                System.Threading.Thread.Sleep(10);
            }


        }

        private void ReadMotionButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.ReadMotion(motionIndex);
        }


        // Playback Motion
        private void preparePlaybackButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.PreparePlaybackMotion(motionIndex, repeating, playbackOption);
        }

        private void startPlaybackButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.StartPlaybackMotionFromPrep();
        }


        private void StopPlaybackButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.StopPlaybackMotion();
        }

        private void PlaybackIntervalSetButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            UInt32 v;
            bool valid = UInt32.TryParse(PlaybackIntervalTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.PlaybackInterval(v);
            }
            else
            {
                MessageBox.Show("Invalid Playback Interval.");
            }
        }


        /*
         * Taskset
         * 
         */
        private void eraseAllTasksetsButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.EraseAllTasksets();
        }

        private void playback1_startButton_Click(object sender, EventArgs e)
        {
            UInt32 v;
            bool valid = UInt32.TryParse(PlaybackIntervalTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.Enable();
                targetMotor.PlaybackInterval(v);
                targetMotor.StartPlaybackMotion(motionIndex, repeating, playbackOption);
            }
            else
            {
                MessageBox.Show("Invalid Playback Interval.");
            }

            
        }

        private void DisableCheckSum_CheckedChanged(object sender, EventArgs e)
        {
            targetMotor.EnableCheckSum(false);
        }

        private void EnableCheckSum_CheckedChanged(object sender, EventArgs e)
        {
            targetMotor.EnableCheckSum(true);

        }

        /*
         * PIDパラメータの設定 
         */

        private void SpeedPButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(SpeedPTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.SpeedPIDP(v);
            }
            else
            {
                MessageBox.Show("Invalid PID Parameter.");
            }
        }

        private void SpeedIButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(SpeedITextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.SpeedPIDI(v);
            }
            else
            {
                MessageBox.Show("Invalid PID Parameter.");
            }
        }
        private void SpeedDButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(SpeedDTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.SpeedPIDD(v);
            }
            else
            {
                MessageBox.Show("Invalid PID Parameter.");
            }
        }



        private void PositionPButtton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(PositionPTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.PositionPIDP(v);
            }
            else
            {
                MessageBox.Show("Invalid PID Parameter.");
            }

        }

        private void positionIButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(PositionITextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.PositionPIDI(v);
            }
            else
            {
                MessageBox.Show("Invalid PID Parameter.");
            }


        }


        private void PositionDButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(PositionDTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.PositionPIDD(v);
            }
            else
            {
                MessageBox.Show("Invalid PID Parameter.");
            }
        }

        private void SaveAllRegistersButton2_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SaveAllRegisters();
        }

        // ここまで。PIDパラメータの設定

        private void ReadDeviceName_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.ReadDeviceName();
        }


        private void SaveAllRegisters_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SaveAllRegisters();
        }

        private void checkedListInterfaceBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }

            byte flag = 0;
            // BLE 
            if (checkedListInterfaceBox.GetItemChecked(0))
            {
                flag |= 1;
                Console.WriteLine("0: {0}",flag);
            }
            // USB
            if (checkedListInterfaceBox.GetItemChecked(1))
            {
                flag |= 0x08; // (3 << 1);
                Console.WriteLine("1: {0}", flag);
            }
            // I2C
            if (checkedListInterfaceBox.GetItemChecked(2))
            {
                flag |= 0x10; // (4 << 1);
                Console.WriteLine("2: {0}", flag);
            }
            // Buttons
            if (checkedListInterfaceBox.GetItemChecked(3))
            {
                flag |= 0x80; // (7 << 1);
                Console.WriteLine("3: {0}", flag);
            }
            targetMotor.Interface(flag);
            //await targetMotor.ReadInterface();
        }

        private void ReadMotorMeasurementButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.ReadMotorMeasurement();
        }


        private void ReadStatusButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.ReadStatus();
        }


        private void ReadAllInfoButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            LogSetText("");
            targetMotor.ReadAllInfo();

        }


        /*
         * グラフ 
         * 
         * 
         */

        private const int max_plot = 200;
        private const double p_max_range = 10800;
        private const double v_max_range = 300;
        private const double t_max_range = 0.6;
        private const double p_trim = p_max_range * 0.1;
        private const double v_trim = v_max_range * 0.1;
        private const double t_trim = t_max_range * 0.1;
        private const double trim_ratio = 0.1;

        private int cnt = 0;
        private void DrawMeasurementGraph()
        {
            BeginInvoke((Action)(() =>
            {
                float p = targetMotor.degree;
                float v = targetMotor.rpm;
                float t = targetMotor.torque;


                // Position
                double p_axisMax = chart1.ChartAreas[0].AxisY.Maximum;
                double p_axisMin = chart1.ChartAreas[0].AxisY.Minimum;
                double p_range = p_axisMax - p_axisMin;

                chart1.ChartAreas[0].AxisY.LabelStyle.Format = "{0:0.00}";
                chart2.ChartAreas[0].AxisY.LabelStyle.Format = "{0:0.00}";
                chart3.ChartAreas[0].AxisY.LabelStyle.Format = "{0:0.00}";


                if (p > p_axisMax)
                {
                    chart1.ChartAreas[0].AxisY.Maximum += p_range * trim_ratio;
                }
                else if (p < p_axisMin)
                {
                    chart1.ChartAreas[0].AxisY.Minimum -= p_range * trim_ratio;

                }
        

                if (chart1.Series[0].Points.Count <= max_plot)
                {
                    chart1.Series[0].Points.AddXY(cnt, p);
                }
                else
                {
                    for (int i = 1; i <= max_plot; i++)
                    {
                        chart1.Series[0].Points[i - 1].YValues = chart1.Series[0].Points[i].YValues;
                    }
                    chart1.Series[0].Points.RemoveAt(max_plot);
                    chart1.Series[0].Points.AddXY(cnt, p);
                }

                // Velocity
                double v_axisMax = chart2.ChartAreas[0].AxisY.Maximum;
                double v_axisMin = chart2.ChartAreas[0].AxisY.Minimum;
                double v_range = v_axisMax - v_axisMin;

                if (v > v_axisMax)
                {
                    chart2.ChartAreas[0].AxisY.Maximum += v_range * trim_ratio;

                }
                else if (v < v_axisMin)
                {
                    chart2.ChartAreas[0].AxisY.Minimum -= v_range * trim_ratio;

                }

                if (chart2.Series[0].Points.Count <= max_plot)
                {
                    chart2.Series[0].Points.AddXY(cnt, v);
                }
                else
                {
                    for (int i = 1; i <= max_plot; i++)
                    {
                        chart2.Series[0].Points[i - 1].YValues = chart2.Series[0].Points[i].YValues;
                    }
                    chart2.Series[0].Points.RemoveAt(max_plot);
                    chart2.Series[0].Points.AddXY(cnt, v);
                }

                // Torque
                double t_axisMax = chart3.ChartAreas[0].AxisY.Maximum;
                double t_axisMin = chart3.ChartAreas[0].AxisY.Minimum;
                double t_range = t_axisMax - t_axisMin;

                if (t > t_axisMax)
                {
                    chart3.ChartAreas[0].AxisY.Maximum += t_range * trim_ratio;

                }
                else if (t < t_axisMin)
                {
                    chart3.ChartAreas[0].AxisY.Minimum -= t_range * trim_ratio;

                }


                if (chart3.Series[0].Points.Count <= max_plot)
                {
                    chart3.Series[0].Points.AddXY(cnt, t);
                }
                else
                {
                    for (int i = 1; i <= max_plot; i++)
                    {
                        chart3.Series[0].Points[i - 1].YValues = chart3.Series[0].Points[i].YValues;
                    }
                    chart3.Series[0].Points.RemoveAt(max_plot);
                    chart3.Series[0].Points.AddXY(cnt, t);
                }

                if (shouldSaveMotorMeasurement)
                {
                    AppendMotorMeasurement(p, v, t);
                }

            }));
        }

        private void AppendMotorMeasurement(float pos, float vel, float trq)
        {


        }

        private void MotorMeasurementSaveButton_Click(object sender, EventArgs e)
        {
            System.Console.WriteLine("MotorMeasurementSaveButton_Click.");
            shouldSaveMotorMeasurement = true;
            var txt = SaveMotorMeasTextBox.Text;

            if (txt.Length == 0)
            {
                System.Console.WriteLine("Input filename.");
                fileName = "test";
            } else
            {
                fileName = txt;
            }

            fileName = fileName + ".csv";

            try
            {
                // appendをtrueにすると，既存のファイルに追記
                //         falseにすると，ファイルを新規作成する
                var append = false;
                // 出力用のファイルを開く
                using (var sw = new System.IO.StreamWriter(fileName, append))
                {
                    System.Console.WriteLine("Write file.");
                    sw.WriteLine("time, degree, rpm, torque");
                    sw.Close();
                }
            }
            catch (System.Exception err)
            {
                // ファイルを開くのに失敗したときエラーメッセージを表示
                System.Console.WriteLine(err.Message);
            }
        }

        private void SetI2CAddressButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            byte v;
            bool valid = byte.TryParse(I2CAddressTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.I2CAddress(v);
            }
            else
            {
                MessageBox.Show("Invalid I2C Address.");
            }
        }

        private void DecrementButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.DecrementPhasediff();
        }

        private void IncrementButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.IncrementPhasediff();

        }

        private void AutoPhasediffAdjustmentButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.AutoPhasediffAdjustment();
        }



        private void SavePhasediffButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SavePhasediff();
        }

        private void SafetySettingsListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }

            byte flag = 0;
            // BLE切断時に 
            if (SafetySettingsListBox.GetItemChecked(0))
            {
                flag |= 1;
                Console.Write("0: {0}", flag);
            }
   
            targetMotor.SafetySettings(flag);
        }

        private void calibrationButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.adcCalibration();
        }




        private void SetKM1Button_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SetMotorModel("KM1");
        }



        private void SetM6829Button_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SetMotorModel("M6829");
        }

        private void SetM4625Button_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SetMotorModel("M4625");
        }

        private void SetM4021Button_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SetMotorModel("M4021");
        }

        private void LedOffButton_CheckedChanged(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.Led(KeiganMotor_USB.LedState.Off, 0, 0, 0);
        }

        private void SetPhaseDiffButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            Int16 pd;
            bool valid = Int16.TryParse(PhaseDiffTextBox.Text.ToString(), out pd);
            if (valid)
            {
                targetMotor.SetPhasediff(pd);
            }
            else
            {
                MessageBox.Show("Invalid Phasediff.");
            }
        }

        private void LPFCurrentButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(LPFCurrentTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.LowPassFilter(KeiganMotor_USB.LowpassFilterType.LPFCurrent, v);
            }
            else
            {
                MessageBox.Show("Invalid Low-pass filter.");
            }
        }

        private void LPFVelocityButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(LPFVelocityTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.LowPassFilter(KeiganMotor_USB.LowpassFilterType.LPFVelocity, v);
            }
            else
            {
                MessageBox.Show("Invalid Low-pass filter.");
            }
        }

        private void LPFPositionButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(LPFPositionTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.LowPassFilter(KeiganMotor_USB.LowpassFilterType.LPFPosition, v);
            }
            else
            {
                MessageBox.Show("Invalid Low-pass filter.");
            }
        }

        private void ReadPIDParamsButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.ReadPIDParams();

        }

        private void PositionIThresholdButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            
            float v;
            bool valid = float.TryParse(PosIThresholdTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.PosControlThreshold( v);
            }
            else
            {
                MessageBox.Show("Invalid PositionIThreshold.");
            }
        }

        byte tableIndex = 0;
        byte paramType = 0;

        private void tableIndexUpDown_ValueChanged(object sender, EventArgs e)
        {
            tableIndex = (byte)tableIndexUpDown.Value;
        }

        private void PIDParamsListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            String text = PIDParamsListBox.SelectedItem.ToString();
            byte _type;
            bool valid = Byte.TryParse(PIDParamsListBox.Text.ToString(), out _type);
            if(valid)
            {
                paramType = _type;
            }
        }

        private void SetPIDTableValueButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(PIDValueTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.SetPIDTableValue(tableIndex, (KeiganMotor_USB.PIDParamType)paramType, v);
                PIDSetText("");
                targetMotor.ReadPIDTable(tableIndex);
            }
            else
            {
                MessageBox.Show("Invalid PIDParam.");
            }
        }

        private void PIDTableSelectButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SelectPIDTable(tableIndex);
        }

        private void ReadPIDTableButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            PIDSetText("");
            targetMotor.ReadPIDTable(tableIndex);
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SavePIDTable();
        }

        private void LimitCurrentButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(LimitCurrentTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.LimitCurrent(v);
            }
            else
            {
                MessageBox.Show("Invalid Limit Current.");
            }
        }

        private void AutoAdjPD_testButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            //targetMotor.AutoPhasediffAdjWOTest();
        }

        private void ResetAllRegistersButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.ResetAllRegisters();
        }

        private UInt32 repeating = 1;
        private KeiganMotor_USB.PlaybackOption playbackOption = 0;

        private void RepeatingButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            UInt32 v;
            bool valid = UInt32.TryParse(RepeatingTextBox.Text.ToString(), out v);
            if (valid)
            {
                repeating = v;
            }
            else
            {
                MessageBox.Show("Invalid repeating times.");
            }
        }

        private void EnterDFUButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.EnterDFU();
        }

        private void PlaybackOptionUpDown_ValueChanged(object sender, EventArgs e)
        {
            playbackOption = (KeiganMotor_USB.PlaybackOption)PlaybackOptionUpDown.Value;
        }

        private void M6829_24V_Button_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SetMotorModel("M6829-24V");
        }

        private void M4625_24V_Button_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SetMotorModel("M4625-24V");
        }

        private void M4021_24V_Button_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SetMotorModel("M4021-24V");
        }

        private void SetBlankModelButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SetMotorModel("blank");
        }

        private void AmpButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }

            float v;
            bool valid = float.TryParse(AmpTextBox.Text.ToString(), out v);
            if (valid)
            {
                ampDeg = v;
            }
            else
            {
                MessageBox.Show("Invalid Amp.");
            }
        }

        private void PresetPositionButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(PresetPositionTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.PresetPositionDegree(v);
            }
            else
            {
                MessageBox.Show("Invalid Position.");
            }
        }

        private void SetUARTBaudRateButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            byte rate = (byte)UARTBaudRateUpDown.Value;
            targetMotor.UARTBaudRate(rate);
        }


        private void SaveAllRegistersButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SaveAllRegisters();
        }

        private void HoldTorqueButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(MaxTorqueTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.HoldTorque(v);
            }
            else
            {
                MessageBox.Show("Invalid Torque.");
            }
        }

        private void PosMaxTextBox_TextChanged(object sender, EventArgs e)
        {
            float v; 
            bool valid = float.TryParse(PosMaxTextBox.Text.ToString(), out v);
            double min = chart1.ChartAreas[0].AxisY.Minimum;
            if (valid && min < v)
            {
                chart1.ChartAreas[0].AxisY.Maximum = v;
            }
        }

        private void PosMinTextBox_TextChanged(object sender, EventArgs e)
        {
            float v;
            bool valid = float.TryParse(PosMinTextBox.Text.ToString(), out v);
            double max = chart1.ChartAreas[0].AxisY.Maximum;
            if (valid && max > v)
            {
                chart1.ChartAreas[0].AxisY.Minimum = v;
            }
        }

        private void VelMaxTextBox_TextChanged(object sender, EventArgs e)
        {
            float v;
            bool valid = float.TryParse(VelMaxTextBox.Text.ToString(), out v);
            double min = chart2.ChartAreas[0].AxisY.Minimum;
            if (valid && min < v)
            {
                chart2.ChartAreas[0].AxisY.Maximum = v;
            }
        }

        private void VelMinTextBox_TextChanged(object sender, EventArgs e)
        {
            float v;
            bool valid = float.TryParse(VelMinTextBox.Text.ToString(), out v);
            double max = chart2.ChartAreas[0].AxisY.Maximum;
            if (valid && max > v)
            {
                chart2.ChartAreas[0].AxisY.Minimum = v;
            }
        }

        private void TrqMaxTextBox_TextChanged(object sender, EventArgs e)
        {
            float v;
            bool valid = float.TryParse(TrqMaxTextBox.Text.ToString(), out v);
            double min = chart3.ChartAreas[0].AxisY.Minimum;
            if (valid && min < v)
            {
                chart3.ChartAreas[0].AxisY.Maximum = v;
            }
        }

        private void TrqMinTextBox_TextChanged(object sender, EventArgs e)
        {
            float v;
            bool valid = float.TryParse(TrqMinTextBox.Text.ToString(), out v);
            double max = chart3.ChartAreas[0].AxisY.Maximum;
            if (valid && max > v)
            {
                chart3.ChartAreas[0].AxisY.Minimum = v;
            }
        }

        private void Factory_Click(object sender, EventArgs e)
        {

        }

        private void MaxTorqueButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            float v;
            bool valid = float.TryParse(MaxTorqueTextBox.Text.ToString(), out v);
            if (valid)
            {
                targetMotor.MaxTorque(v);
            }
            else
            {
                MessageBox.Show("Invalid Torque.");
            }
        }

        private void ReadRegTestButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.ReadRegister(0x1E);
        }

        private void PosArrivalTestButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            isPosArrivalTestEnabled = true;
            targetMotor.Enable();
            targetMotor.SpeedRpm(300);
            targetMotor.MoveByDistanceDegree(360);
        }

        private void M2813_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.SetMotorModel("M2813");
        }

        private void OufukuSyncButton_Click(object sender, EventArgs e)
        {

        }

        private async void ACCButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            Double lpf = 0.5;
            Double old_v = 0;

            for (int t = 0; t < 50; t++)
            {
                //Double v = (1 - lpf) * old_v + lpf * array[t];
                //targetMotor.RunAtVelocity((float)v);
                targetMotor.RunAtVelocity(-20);
                //old_v = v;
                //System.Threading.Thread.Sleep(100);
                await Task.Delay(100);
                targetMotor.RunAtVelocity(-10);
                await Task.Delay(100);
            }
            targetMotor.Free();
        }

        private Byte motorIntvlFlag = 3;

        private void MotorMeasIntvlNumericUpDown_ValueChanged(object sender, EventArgs e)
        {
            motorIntvlFlag = (Byte)MotorMeasIntvlNumericUpDown.Value;
        }

        private void MotorMeasIntvlButton_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.MotorMeasurementInterval(motorIntvlFlag);
        }

        private void ExecBurn_Click(object sender, EventArgs e)
        {
            if (!TargetMotorExists()) { return; }
            targetMotor.BurnPhasediff();
        }

        private void SpeedTextBox_TextChanged(object sender, EventArgs e)
        {

        }
    }
}
