﻿using System;
using System.Linq;
using System.IO.Ports;


/* KeiganMotor API
 * 
 * Factory only
 * 
 */

public partial class KeiganMotor_USB
{
    // 自動位相調整を自動で実行する
    public void AutoPhasediffAdjustment()
    {
        byte[] param = { 0xFB, 0x00, 0x00, // コマンド, ID
            0x00, 0x00, 0xB0, 0x1C, 0x43, 0x10, 0x99, 0x21, 0x21, // magic number
            0x00, 0x00 }; // CRC 任意

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, 14);
        }
    }

    // 全方向回りながら、自動位相調整を実行する
    public void AutoPhasediffAdjustmentOmni()
    {
        byte[] param = { 0xFB, 0x00, 0x00, // コマンド, ID
            0xA0, 0x00, 0xB0, 0x1C, 0x43, 0x10, 0x99, 0x21, 0x21, // magic number
            0x00, 0x00 }; // CRC 任意

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, 14);
        }
    }


    // 全方向回りながら、自動位相調整を実行し、エンコーダ書き込み待ちとする
    public void AutoPhasediffAdjustmentOmniBurn()
    {
        byte[] param = { 0xFB, 0x00, 0x00, // コマンド, ID
            0xB0, 0x00, 0xB0, 0x1C, 0x43, 0x10, 0x99, 0x21, 0x21, // magic number
            0x00, 0x00 }; // CRC 任意

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, 14);
        }
    }

    // 位相調整値 +10 する（saveAllRegistersを実行で保存）
    public void IncrementPhasediff()
    {
        byte[] param = { 0xFB, 0x00, 0x00, // コマンド, ID
            0x01, 0x00, 0xB0, 0x1C, 0x43, 0x10, 0x99, 0x21, 0x21, // magic number
            0x00, 0x00 }; // CRC 任意

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, 14);
        }
    }

    // 位相調整値 -10 する（saveAllRegistersを実行で保存）
    public void DecrementPhasediff()
    {
        byte[] param = { 0xFB, 0x00, 0x00, // コマンド, ID
            0x02, 0x00, 0xB0, 0x1C, 0x43, 0x10, 0x99, 0x21, 0x21, // magic number
            0x00, 0x00 }; // CRC 任意

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, 14);
        }

    }

    public void SetPhasediff(Int16 pd)
    {
        byte[] param = { 0xFB, 0x00, 0x00, // コマンド, ID
            0x20, // magic number
            0x00, 0x00, // phaseDiff
            0x1C, 0x43, 0x10, 0x99, 0x21, 0x21, // magic number
            0x00, 0x00 }; // CRC 任意

        byte[] pdArray = Int16ToBytes(pd);
        pdArray.CopyTo(param, 4);

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, param.Length);
        }

    }

    // 位相調整値 フラッシュメモリに書き込む
    public void SavePhasediff()
    {
        byte[] param = { 0xFB, 0x00, 0x00, // コマンド, ID
            0x10, 0x00, 0xB0, 0x1C, 0x43, 0x10, 0x99, 0x21, 0x21, // magic number
            0x00, 0x00 }; // CRC 任意

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, 14);
        }

    }

    // 位相調整値情報を含むゼロ点とモーター型番情報をエンコーダに書き込む
    public void BurnPhasediff()
    {
        byte[] param = { 0xFB, 0x00, 0x00, // コマンド, ID
            0x30, 0x00, 0xB0, 0x1C, 0x43, 0x10, 0x99, 0x21, 0x21, // magic number
            0x00, 0x00 }; // CRC 任意

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, 14);
        }

    }

    // 位相調整値 フラッシュメモリに書き込む
    public void SetMotorModel(String motorModel)
    {
        byte[] param = { 0xFC, 0x00, 0x00, // コマンド, ID
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // motor_model
            0x10, 0x99, 0x21, 0x21, // magic number
            0x00, 0x00 }; // CRC 任意

        //ASCII エンコード
        byte[] model = System.Text.Encoding.ASCII.GetBytes(motorModel);

        model.CopyTo(param, 3);

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, param.Length);
        }

    }

    public void adcCalibration()
    {
        byte[] param = new byte[5];
        int size = 0;

        //パラメータのセット
        param[size++] = 0x99; // adcCalibration コマンド

        GetIDBytes().CopyTo(param, size);
        size += 2;


        param[size++] = 0; //CRC16 任意でOK
        param[size++] = 0; //CRC16 任意でOK

        if (serialPort.IsOpen == true)
        {
            serialPort.Write(param, 0, size);
        }
    }

}
