﻿using System;
using System.Linq;
using System.IO.Ports;
using System.Runtime.InteropServices.WindowsRuntime;

public partial class KeiganMotor_USB
{
    public SerialPort serialPort;
    public UInt16 commandID;
    public delegate void DidReadStatus(KeiganMotor_USB motor, Status status);
    public delegate void DidReceivedError(KeiganMotor_USB motor, UInt16 id, byte cmd, ErrorCode code, UInt32 info);
    public delegate void DidReceivedPosArrival(KeiganMotor_USB motor, UInt16 id, float arrivalPos);
    public delegate void DidReadFloat(KeiganMotor_USB motor, UInt16 id, byte register, float value);
    public delegate void DidReadPIDTable(KeiganMotor_USB motor, UInt16 id, byte tableIndex, PIDParamType type, float value);
    public delegate void DidReadText(KeiganMotor_USB motor, String text);
    public delegate void DidReadMotion(KeiganMotor_USB motor, UInt16 id, UInt32 totalPosNum, UInt32 posIndex, UInt32 posNum, float[] posArray);

    // TODO public delegate void DidReceivedError(KeiganMotor_USB motor, UInt16 id, byte command, byte errorCode, UInt32 infomation);
    public delegate void DidUpdateIMUMeasurement(KeiganMotor_USB motor, Int16 accelX, Int16 accelY, Int16 accelZ, Int16 temp, Int16 gyroX, Int16 gyroY, Int16 gyroZ);
    public delegate void DidUpdateMotorMeasurement(KeiganMotor_USB motor, float position, float velocity, float torque);

    public event DidReadFloat ReadFloatEvent = delegate (KeiganMotor_USB motor, UInt16 id, byte register, float value) { };
    public event DidReadPIDTable ReadPIDTableEvent = delegate (KeiganMotor_USB motor, UInt16 id, byte tableIndex, PIDParamType type, float value) { };
    public event DidUpdateMotorMeasurement MotorMeasurementEvent = delegate (KeiganMotor_USB motor, float position, float velocity, float torque) { };
    public event DidUpdateIMUMeasurement IMUMeasurementEvent = delegate (KeiganMotor_USB motor, Int16 accelX, Int16 accelY, Int16 accelZ, Int16 temp, Int16 gyroX, Int16 gyroY, Int16 gyroZ) { };
    public event DidReadStatus ReadStatusEvent = delegate (KeiganMotor_USB motor, Status status) { };
    public event DidReceivedError ReceivedErrorEvent = delegate (KeiganMotor_USB motor, UInt16 id, byte cmd, ErrorCode error, UInt32 info) { };
    public event DidReceivedPosArrival ReceivedPosArrivalEvent = delegate (KeiganMotor_USB motor, UInt16 id, float arrivalPos) { };
    public event DidReadText ReadTextEvent = delegate (KeiganMotor_USB motor, String text) { };
    public event DidReadMotion ReadMotionEvent = delegate (KeiganMotor_USB motor, UInt16 id, UInt32 totalPosNum, UInt32 posIndex, UInt32 posNum, float[] posArray) { };

    public InterfaceSetting interfaceSetting = new InterfaceSetting();
    public string deviceName = "unknown";
    public string deviceInfo = "unknown";
    public string versionText = "unknown";
    public float versionNumber = 0;

    public float acc;
    public float dec;
    public float speedP;
    public float speedI;
    public float speedD;
    public float positionP;
    public float positionI;
    public float positionD;
    public float positionPIDThreshold;

    // 位置到達の誤差と判定時間
    public bool notifyPosArrival;
    public float posArrivalTolerance;
    public UInt32 posArrivalJudgeTime;

    // RADIO
    public bool radioEnabled;

    public float position;
    public float degree;
    public float velocity;
    public float rpm;
    public float torque;

    public Int16 accelX, accelY, accelZ, temp, gyroX, gyroY, gyroZ;

    // PID の種類
    public enum ErrorCode
    {
        KM_SUCCESS = 0,
        KM_ERROR_INTERNAL = 1,
        KM_ERROR_NO_MEM = 2,
        KM_ERROR_DATA_NOT_FOUND = 3,
        KM_ERROR_NOT_SUPPORTED = 4,
        KM_ERROR_INVALID_COMMAND = 5,
        KM_ERROR_INVALID_PARAM = 6,
        KM_ERROR_STORAGE_FULL = 7,
        KM_ERROR_INVALID_FLASH_STATE = 8,
        KM_ERROR_INVALID_LENGTH = 9,
        KM_ERROR_INVALID_CHECKSUM = 10,
        KM_ERROR_TIMEOUT = 13,
        KM_ERROR_FORBIDDEN = 15,
        KM_ERROR_INVALID_ADDR = 16,
        KM_ERROR_BUSY = 17,
        KM_ERROR_RESOURCE = 18,
        KM_ERROR_MOTOR_DISABLED = 20,
        KM_ERROR_DEVICE_DRIVER = 60,
        KM_ERROR_DEVICE_FLASH = 61,
        KM_ERROR_DEVICE_LED = 62,
        KM_ERROR_DEVICE_IMU = 63,
        KM_ERROR_NRF_DEVICE = 70,
        KM_ERROR_WDT_EVENT = 80,
        KM_ERROR_OVER_HEAT = 81,
        KM_SUCCESS_ARRIVAL = 100

    }

    public class Status
    {
        public bool isEnabled;
        public bool isQueuePaused;
        public bool isMotorMeasurementEnabled;
        public bool isIMUMeasurementEnabled;
        public bool isCheckSumEnabled;
        public byte flashMemoryState;
        public byte motorControlMode;

        public Status()
        {
            reset();
        }

        public void reset()
        {
            isEnabled = false;
            isQueuePaused = false;
            isMotorMeasurementEnabled = true;
            isIMUMeasurementEnabled = false;
            isCheckSumEnabled = false;
            flashMemoryState = 0;
            motorControlMode = 0;
        }
    }

    public Status status;

    public KeiganMotor_USB(string portName, int baudRate = 115200)
    {
        this.serialPort = new SerialPort();
        // KeiganMotor KM-1 needs the following UART settings.
        serialPort.PortName = portName;
        serialPort.BaudRate = baudRate;
        serialPort.DataBits = 8;
        serialPort.DtrEnable = true;
        serialPort.Handshake = Handshake.RequestToSend;
        serialPort.Parity = Parity.None;
        serialPort.RtsEnable = true;
        serialPort.StopBits = StopBits.One;

        serialPort.DataReceived += new SerialDataReceivedEventHandler(DataReceivedHandler);
        CRC16Init();

    }


    public void DataReceivedHandler(
                    object sender,
                    SerialDataReceivedEventArgs e)
    {
        DateTime t = DateTime.Now;
        //Console.WriteLine("Received: {0}", t.Millisecond);
        SerialPort sp = (SerialPort)sender;
        int bytes = sp.BytesToRead;
        byte[] buffer = new byte[bytes];
        sp.Read(buffer, 0, bytes);
        HandleReceivedData(buffer);

    }

    public void HandleReceivedData(byte[] buf)
    {
        //Console.WriteLine(BitConverter.ToString(buf));
        //Console.WriteLine(buf.Length);

        if (buf.Length > 0)
        {
            String str = System.Text.Encoding.ASCII.GetString(buf);
            //Console.Write(str);

            ReadTextEvent(this, str);

        }
        
        for (int i = 0; i < buf.Length; i++)
        {
            //Console.WriteLine(i);
            HandleReceivedByte(buf[i]);

        }
    }



    public bool Open()
    {
        try
        {
            serialPort.Open();
        }
        catch (Exception ex)
        {
            Console.WriteLine(ex.Message);
            return false;
        }
        return true;
    }


    public bool Close()
    {
        try
        {
            serialPort.Close();
        }
        catch (Exception ex)
        {
            Console.WriteLine(ex.Message);
            return false;
        }
        return true;
    }



    public byte[] GetIDBytes()
    {
        byte[] a = UInt16ToBytes(this.commandID);

        if (this.commandID < 0xFFFF)
        {
            this.commandID++;
        }
        else
        {
            this.commandID = 0;
        }
        return a;
    }




}
